import javax.swing.*;

import java.awt.*;
import java.awt.event.*;

import kwm.*;

public class InteractiveBall 
{
  // delay in milliseconds (1000/DELAY frames per second)
  private static final int DELAY = 40;

  // width and height of drawing area
  private static final int WIDTH = 600;
  private static final int HEIGHT = 400;

  // radius and color of ball
  private static final int RADIUS = 15;

  // the speedup/slow down and the minimal speed
  private static final double SPEEDUP = 1.1;
  private static final double MINSPEED = 3;

  public static void main(String[] args)
  {
    // open window
    Drawing.begin("Interactive Ball", WIDTH, HEIGHT);

    // create and register listener
    BallListener listener = new BallListener();
    Drawing.frame.addKeyListener(listener);
    Drawing.panel.addMouseListener(listener);
    Drawing.panel.addMouseMotionListener(listener);

    // run program until window is closed
    Timer timer = new Timer(DELAY, listener);
    timer.start(); 
    try 
    { 
      synchronized(timer) { timer.wait(); }
    }
    catch(InterruptedException e)
    { }
  }

  private static class BallListener implements 
  ActionListener, KeyListener, MouseListener, MouseMotionListener
  {
    // position and velocity of ball
    private int x = 117;
    private int y = 63;
    private double vx = 17;
    private double vy = 13;

    // true if ball is stopped
    private boolean stopped = false;

    // execute animation step
    public void actionPerformed(ActionEvent e) 
    {
      // update model
      if (!stopped)
      {
        x += vx;
        y += vy;
        if (x <= RADIUS) { x = RADIUS; vx = -vx; }
        if (y <= RADIUS) { y = RADIUS; vy = -vy; }
        if (x >= WIDTH-RADIUS)  { x = WIDTH-RADIUS;  vx = -vx; }
        if (y >= HEIGHT-RADIUS) { y = HEIGHT-RADIUS; vy = -vy; }
      }

      // draw model
      Drawing.graphics.clearRect(0, 0, WIDTH, HEIGHT);
      Drawing.graphics.setColor(Color.orange);
      Drawing.graphics.fillOval(x-RADIUS, y-RADIUS, 2*RADIUS, 2*RADIUS);
      Drawing.graphics.setColor(Color.black);
      Drawing.graphics.drawOval(x-RADIUS, y-RADIUS, 2*RADIUS, 2*RADIUS);
      Drawing.paint();
    }

    // handle key events
    public void keyTyped(KeyEvent e) { }
    public void keyPressed(KeyEvent e) 
    { 
      switch (e.getKeyCode())
      {
      case KeyEvent.VK_UP:   
        // speedup ball on pressing arrow up key
        vx = vx*SPEEDUP; if (Math.abs(vx) < MINSPEED) vx = MINSPEED;
        vy = vy*SPEEDUP; if (Math.abs(vy) < MINSPEED) vy = MINSPEED;
        break;
      case KeyEvent.VK_DOWN: 
        // slow down ball on pressing arrow down key
        vx = vx/SPEEDUP; if (Math.abs(vx) < MINSPEED) vx = 0; 
        vy = vy/SPEEDUP; if (Math.abs(vy) < MINSPEED) vy = 0;
        break;
      }
    }
    public void keyReleased(KeyEvent e) { }

    // handle mouse events
    public void mousePressed(MouseEvent e) 
    { 
      // stop movement and update position when mouse button pressed
      stopped = true;
      x = e.getX();
      y = e.getY();
    }
    public void mouseReleased(MouseEvent e) 
    { 
      // restart movement when mouse button is released
      stopped = false;
    }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }
    public void mouseClicked(MouseEvent e) { }

    // handle mouse motion events
    public void mouseMoved(MouseEvent e) { }
    public void mouseDragged(MouseEvent e)
    {
      // update position when mouse is moved while button is pressed
      x = e.getX();
      y = e.getY();
    }
  }
}
